import spirit.spiritlib as spiritlib
import ctypes

### Load Library
_spirit = spiritlib.LoadSpiritLibrary()

### Read an image from disk
_Image_Read             = _spirit.IO_Image_Read
_Image_Read.argtypes    = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_int, ctypes.c_int, 
                           ctypes.c_int]
_Image_Read.restype     = None
def Image_Read(p_state, filename, idx_image_infile=0, idx_image_inchain=-1, idx_chain=-1):
    _Image_Read(ctypes.c_void_p(p_state), ctypes.c_char_p(filename.encode('utf-8')), 
                ctypes.c_int(idx_image_infile), ctypes.c_int(idx_image_inchain), 
                ctypes.c_int(idx_chain))

### Write an image to disk
_Image_Write             = _spirit.IO_Image_Write
_Image_Write.argtypes    = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_int, 
                            ctypes.c_char_p, ctypes.c_int, ctypes.c_int]
_Image_Write.restype     = None
def Image_Write(p_state, filename, fileformat=6, comment=" ", idx_image=-1, idx_chain=-1):
    _Image_Write(ctypes.c_void_p(p_state), ctypes.c_char_p(filename.encode('utf-8')), 
                 ctypes.c_int(fileformat), ctypes.c_char_p(comment.encode('utf-8')),
                 ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Append an image to an existing file
_Image_Append             = _spirit.IO_Image_Append
_Image_Append.argtypes    = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_int, 
                             ctypes.c_char_p, ctypes.c_int, ctypes.c_int]
_Image_Append.restype     = None
def Image_Append(p_state, filename, fileformat=6, comment=" ", idx_image=-1, idx_chain=-1):
    _Image_Append(ctypes.c_void_p(p_state), ctypes.c_char_p(filename.encode('utf-8')), 
                  ctypes.c_int(fileformat), ctypes.c_char_p(filename.encode('utf-8')), 
                  ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Read a chain of images from disk
_Chain_Read             = _spirit.IO_Chain_Read
_Chain_Read.argtypes    = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_int, ctypes.c_int, 
                           ctypes.c_int, ctypes.c_int, ctypes.c_int]
_Chain_Read.restype     = None
def Chain_Read(p_state, filename, fileformat=6, starting_image=-1, ending_image=-1, insert_idx=-1, 
               idx_chain=-1):
    _Chain_Read(ctypes.c_void_p(p_state), ctypes.c_char_p(filename.encode('utf-8')), 
                ctypes.c_int(fileformat), ctypes.c_int(starting_image), ctypes.c_int(ending_image), 
                ctypes.c_int(insert_idx), ctypes.c_int(idx_chain))

### Write a chain of images to disk
_Chain_Write             = _spirit.IO_Chain_Write
_Chain_Write.argtypes    = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_int, ctypes.c_char_p, 
                            ctypes.c_int]
_Chain_Write.restype     = None
def Chain_Write(p_state, filename, fileformat=6, comment=" ", idx_chain=-1):
    _Chain_Write(ctypes.c_void_p(p_state), ctypes.c_char_p(filename.encode('utf-8')),
                 ctypes.c_int(fileformat), ctypes.c_char_p(comment.encode('utf-8')), 
                 ctypes.c_int(idx_chain))

### Append a chain of images to disk
_Chain_Append             = _spirit.IO_Chain_Append
_Chain_Append.argtypes    = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_int, ctypes.c_char_p, 
                            ctypes.c_int]
_Chain_Append.restype     = None
def Chain_Append(p_state, filename, fileformat=6, comment=" ", idx_chain=-1):
    _Chain_Append(ctypes.c_void_p(p_state), ctypes.c_char_p(filename.encode('utf-8')),
                 ctypes.c_int(fileformat), ctypes.c_char_p(comment.encode('utf-8')), 
                 ctypes.c_int(idx_chain))




import spirit.spiritlib as spiritlib
import ctypes

### Load Library
_spirit = spiritlib.LoadSpiritLibrary()

## ---------------------------------- Set ----------------------------------

### Set the output file tag, which is placed in front of all output files of MC simulations
_Set_MC_Output_Tag          = _spirit.Parameters_Set_MC_Output_Tag
_Set_MC_Output_Tag.argtypes = [ctypes.c_void_p, ctypes.c_char_p,
                                ctypes.c_int, ctypes.c_int]
_Set_MC_Output_Tag.restype  = None
def setOutputTag(p_state, tag, idx_image=-1, idx_chain=-1):
    _Set_MC_Output_Tag(ctypes.c_void_p(p_state), ctypes.c_char_p(tag.encode('utf-8')),
                        ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set the output folder for MC simulations
_Set_MC_Output_Folder          = _spirit.Parameters_Set_MC_Output_Folder
_Set_MC_Output_Folder.argtypes = [ctypes.c_void_p, ctypes.c_char_p,
                                    ctypes.c_int, ctypes.c_int]
_Set_MC_Output_Folder.restype  = None
def setOutputFolder(p_state, folder, idx_image=-1, idx_chain=-1):
    _Set_MC_Output_Folder(ctypes.c_void_p(p_state), ctypes.c_char_p(folder.encode('utf-8')),
                        ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set the output configuration
_Set_MC_Output_General          = _spirit.Parameters_Set_MC_Output_General
_Set_MC_Output_General.argtypes = [ctypes.c_void_p, ctypes.c_bool, ctypes.c_bool,
                                    ctypes.c_bool, ctypes.c_int, ctypes.c_int]
_Set_MC_Output_General.restype  = None
def setOutputGeneral(p_state, any=True, initial=False, final=False, idx_image=-1, idx_chain=-1):
    _Set_MC_Output_General(ctypes.c_void_p(p_state), ctypes.c_bool(any), ctypes.c_bool(initial),
                        ctypes.c_bool(final), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set the output configuration for energy files
_Set_MC_Output_Energy          = _spirit.Parameters_Set_MC_Output_Energy
_Set_MC_Output_Energy.argtypes = [ctypes.c_void_p, ctypes.c_bool, ctypes.c_bool,
                                    ctypes.c_bool, ctypes.c_bool,
                                    ctypes.c_bool, ctypes.c_int, ctypes.c_int]
_Set_MC_Output_Energy.restype  = None
def setOutputEnergy(p_state, step=False, archive=True, spin_resolved=False, divide_by_nos=True, add_readability_lines=True, idx_image=-1, idx_chain=-1):
    _Set_MC_Output_Energy(ctypes.c_void_p(p_state), ctypes.c_bool(step), ctypes.c_bool(archive),
                        ctypes.c_bool(spin_resolved), ctypes.c_bool(divide_by_nos), ctypes.c_bool(add_readability_lines),
                        ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set the output configuration for energy files
_Set_MC_Output_Configuration          = _spirit.Parameters_Set_MC_Output_Configuration
_Set_MC_Output_Configuration.argtypes = [ctypes.c_void_p, ctypes.c_bool, ctypes.c_bool,
                                            ctypes.c_int, ctypes.c_int, ctypes.c_int]
_Set_MC_Output_Configuration.restype  = None
def setOutputConfiguration(p_state, step=False, archive=True, filetype=6, idx_image=-1, idx_chain=-1):
    _Set_MC_Output_Configuration(ctypes.c_void_p(p_state), ctypes.c_bool(step), ctypes.c_bool(archive),
                        ctypes.c_int(filetype), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set number of iterations and step size
_Set_MC_N_Iterations             = _spirit.Parameters_Set_MC_N_Iterations
_Set_MC_N_Iterations.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int,
                                     ctypes.c_int, ctypes.c_int]
_Set_MC_N_Iterations.restype     = None
def setIterations(p_state, n_iterations, n_iterations_log, idx_image=-1, idx_chain=-1):
    _Set_MC_N_Iterations(ctypes.c_void_p(p_state), ctypes.c_int(n_iterations),
                          ctypes.c_int(n_iterations_log), ctypes.c_int(idx_image),
                          ctypes.c_int(idx_chain))

### Set temperature
_Set_MC_Temperature             = _spirit.Parameters_Set_MC_Temperature
_Set_MC_Temperature.argtypes    = [ctypes.c_void_p, ctypes.c_float, ctypes.c_int, ctypes.c_int]
_Set_MC_Temperature.restype     = None
def setTemperature(p_state, temperature, idx_image=-1, idx_chain=-1):
    _Set_MC_Temperature(ctypes.c_void_p(p_state), ctypes.c_float(temperature),
                         ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set target acceptance ratio
_Set_MC_Acceptance_Ratio             = _spirit.Parameters_Set_MC_Acceptance_Ratio
_Set_MC_Acceptance_Ratio.argtypes    = [ctypes.c_void_p, ctypes.c_float, ctypes.c_int, ctypes.c_int]
_Set_MC_Acceptance_Ratio.restype     = None
def setAcceptanceRatio(p_state, ratio, idx_image=-1, idx_chain=-1):
    _Set_MC_Acceptance_Ratio(p_state, ctypes.c_float(ratio), idx_image, idx_chain)

## ---------------------------------- Get ----------------------------------

### Get number of iterations and step size
_Get_MC_N_Iterations             = _spirit.Parameters_Get_MC_N_Iterations
_Get_MC_N_Iterations.argtypes    = [ctypes.c_void_p, ctypes.POINTER( ctypes.c_int ),
                                     ctypes.POINTER( ctypes.c_int ), ctypes.c_int, ctypes.c_int]
_Get_MC_N_Iterations.restype     = None
def getIterations(p_state, idx_image=-1, idx_chain=-1):
    n_iterations = ctypes.c_int()
    n_iterations_log = ctypes.c_int()
    _Get_MC_N_Iterations(p_state, ctypes.pointer(n_iterations), ctypes.pointer(n_iterations_log),
                          ctypes.c_int(idx_image), ctypes.c_int(idx_chain))
    return int(n_iterations.value), int(n_iterations_log.value)

### Get temperature
_Get_MC_Temperature             = _spirit.Parameters_Get_MC_Temperature
_Get_MC_Temperature.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Get_MC_Temperature.restype     = ctypes.c_float
def getTemperature(p_state, idx_image=-1, idx_chain=-1):
    return float(_Get_MC_Temperature(ctypes.c_void_p(p_state), ctypes.c_int(idx_image),
                                             ctypes.c_int(idx_chain)))

### Get target acceptance ratio
_Get_MC_Acceptance_Ratio             = _spirit.Parameters_Get_MC_Acceptance_Ratio
_Get_MC_Acceptance_Ratio.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Get_MC_Acceptance_Ratio.restype     = ctypes.c_float
def getAcceptanceRatio(p_state, idx_image=-1, idx_chain=-1):
    return float(_Get_MC_Acceptance_Ratio(ctypes.c_void_p(p_state), ctypes.c_int(idx_image),
                                    ctypes.c_int(idx_chain)))


import spirit.spiritlib as spiritlib
import spirit.parameters as parameters
import ctypes

### Load Library
_spirit = spiritlib.LoadSpiritLibrary()


### Get Chain index
_Get_Index          = _spirit.Chain_Get_Index
_Get_Index.argtypes = [ctypes.c_void_p]
_Get_Index.restype  = ctypes.c_int
def Get_Index(p_state):
    return int(_Get_Index(ctypes.c_void_p(p_state)))


### Get Chain number of images
_Get_NOI            = _spirit.Chain_Get_NOI
_Get_NOI.argtypes   = [ctypes.c_void_p, ctypes.c_int]
_Get_NOI.restype    = ctypes.c_int
def Get_NOI(p_state, idx_chain=-1):
    return int(_Get_NOI(ctypes.c_void_p(p_state), ctypes.c_int(idx_chain)))


### Switch active to next image of chain
_next_Image             = _spirit.Chain_next_Image
_next_Image.argtypes    = [ctypes.c_void_p, ctypes.c_int]
_next_Image.restype     = ctypes.c_bool
def Next_Image(p_state, idx_chain=-1):
    return bool(_next_Image(ctypes.c_void_p(p_state), ctypes.c_int(idx_chain)))

### Switch active to previous image of chain
_prev_Image             = _spirit.Chain_prev_Image
_prev_Image.argtypes    = [ctypes.c_void_p, ctypes.c_int]
_prev_Image.restype     = ctypes.c_bool
def Prev_Image(p_state, idx_chain=-1):
    return bool(_prev_Image(ctypes.c_void_p(p_state), ctypes.c_int(idx_chain)))

### Switch active to specific image of chain
_Jump_To_Image             = _spirit.Chain_Jump_To_Image
_Jump_To_Image.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Jump_To_Image.restype     = ctypes.c_bool
def Jump_To_Image(p_state, idx_image=-1, idx_chain=-1):
    return bool(_Jump_To_Image(ctypes.c_void_p(p_state), ctypes.c_int(idx_image), ctypes.c_int(idx_chain)))


### Copy active image to clipboard
_Image_to_Clipboard             = _spirit.Chain_Image_to_Clipboard
_Image_to_Clipboard.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Image_to_Clipboard.restype     = None
def Image_to_Clipboard(p_state, idx_image=-1, idx_chain=-1):
    _Image_to_Clipboard(ctypes.c_void_p(p_state), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Replace active image in chain
_Replace_Image             = _spirit.Chain_Replace_Image
_Replace_Image.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Replace_Image.restype     = None
def Replace_Image(p_state, idx_image=-1, idx_chain=-1):
    _Replace_Image(ctypes.c_void_p(p_state), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))


### Insert clipboard image before image in chain
_Insert_Image_Before             = _spirit.Chain_Insert_Image_Before
_Insert_Image_Before.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Insert_Image_Before.restype     = None
def Insert_Image_Before(p_state, idx_image=-1, idx_chain=-1):
    _Insert_Image_Before(ctypes.c_void_p(p_state), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Insert clipboard image after image in chain
_Insert_Image_After             = _spirit.Chain_Insert_Image_After
_Insert_Image_After.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Insert_Image_After.restype     = None
def Insert_Image_After(p_state, idx_image=-1, idx_chain=-1):
    _Insert_Image_After(ctypes.c_void_p(p_state), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Insert clipboard image at end of chain
_Push_Back             = _spirit.Chain_Push_Back
_Push_Back.argtypes    = [ctypes.c_void_p, ctypes.c_int]
_Push_Back.restype     = None
def Push_Back(p_state, idx_chain=-1):
    _Push_Back(ctypes.c_void_p(p_state), ctypes.c_int(idx_chain))



### Delete active image
_Delete_Image             = _spirit.Chain_Delete_Image
_Delete_Image.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Delete_Image.restype     = ctypes.c_bool
def Delete_Image(p_state, idx_image=-1, idx_chain=-1):
    return bool(_Delete_Image(ctypes.c_void_p(p_state), ctypes.c_int(idx_image), ctypes.c_int(idx_chain)))

### Delete image at end of chain
_Pop_Back             = _spirit.Chain_Pop_Back
_Pop_Back.argtypes    = [ctypes.c_void_p, ctypes.c_int]
_Pop_Back.restype     = ctypes.c_bool
def Pop_Back(p_state, idx_chain=-1):
    return bool(_Pop_Back(ctypes.c_void_p(p_state), ctypes.c_int(idx_chain)))


### Update the chain's data (interpolated energies etc.)
_Update_Data             = _spirit.Chain_Update_Data
_Update_Data.argtypes    = [ctypes.c_void_p, ctypes.c_int]
_Update_Data.restype     = None
def Update_Data(p_state, idx_chain=-1):
    _Update_Data(ctypes.c_void_p(p_state), ctypes.c_int(idx_chain))


### Setup the chain's data arrays (when is this necessary?)
_Setup_Data             = _spirit.Chain_Setup_Data
_Setup_Data.argtypes    = [ctypes.c_void_p, ctypes.c_int]
_Setup_Data.restype     = None
def Setup_Data(p_state, idx_chain=-1):
    _Setup_Data(ctypes.c_void_p(p_state), ctypes.c_int(idx_chain))


### Get Rx
_Get_Rx          = _spirit.Chain_Get_Rx
_Get_Rx.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_float), ctypes.c_int]
_Get_Rx.restype  = None
def Get_Rx(p_state, idx_chain=-1):
    noi = Get_NOI(p_state, idx_chain)
    arrayX = ctypes.c_float * noi
    Rx = [0]*noi
    _Rx = arrayX(*Rx)
    _Get_Rx(ctypes.c_void_p(p_state), _Rx, ctypes.c_int(idx_chain))
    for i in range(noi):
        Rx[i] = _Rx[i]
    return Rx

### Get Rx interpolated
_Get_Rx_Interpolated          = _spirit.Chain_Get_Rx_Interpolated
_Get_Rx_Interpolated.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_float), ctypes.c_int]
_Get_Rx_Interpolated.restype  = None
def Get_Rx_Interpolated(p_state, idx_chain=-1):
    noi = Get_NOI(p_state, idx_chain)
    n_interp = parameters.gneb.getEnergyInterpolations(p_state, idx_chain)
    len_Rx = noi + (noi-1)*n_interp
    Rx = (len_Rx*ctypes.c_float)()
    _Get_Rx_Interpolated(ctypes.c_void_p(p_state), Rx, ctypes.c_int(idx_chain))
    return Rx

### Get Energy
_Get_Energy          = _spirit.Chain_Get_Energy
_Get_Energy.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_float), ctypes.c_int]
_Get_Energy.restype  = None
def Get_Energy(p_state, idx_chain=-1):
    noi = Get_NOI(p_state, idx_chain)
    Energy = (noi*ctypes.c_float)()
    _Get_Energy(ctypes.c_void_p(p_state), Energy, ctypes.c_int(idx_chain))
    return Energy

### Get Energy Interpolated
_Get_Energy_Interpolated          = _spirit.Chain_Get_Energy_Interpolated
_Get_Energy_Interpolated.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_float), ctypes.c_int]
_Get_Energy_Interpolated.restype  = None
def Get_Energy_Interpolated(p_state, idx_chain=-1):
    noi = Get_NOI(p_state, idx_chain)
    n_interp = parameters.gneb.getEnergyInterpolations(p_state, idx_chain)
    len_Energy = noi + (noi-1)*n_interp
    Energy_interp = (len_Energy*ctypes.c_float)()
    _Get_Energy_Interpolated(ctypes.c_void_p(p_state), Energy_interp, ctypes.c_int(idx_chain))
    return Energy_interp


### Get the operating system and according library name
def GetSpiritLibName():
    from sys import platform as _platform
    libname = ''
    if _platform == "linux" or _platform == "linux2":
        # Linux
        libname = 'libSpirit.so'
    elif _platform == "darwin":
        # OS X
        libname = 'libSpirit.dylib'
    elif _platform == "win32":
        # Windows
        libname = 'Spirit.dll'
    return libname

### Get the Spirit library as CDLL
def LoadSpiritLibrary():
    import os
    import ctypes

    ### Get this file's directory. The library should be here
    spirit_py_dir = os.path.dirname(os.path.realpath(__file__))

    libname = GetSpiritLibName()

    ### Load the Spirit library
    _spirit = ctypes.CDLL(spirit_py_dir + '/' + libname)

    ### Return
    return _spirit

### Wrap a function in a thread for it to be interruptible
def WrapFunction(function, arguments):
    import threading
    t = threading.Thread(target=function, args=arguments)
    t.daemon = True
    t.start()
    while t.is_alive(): # wait for the thread to exit
        t.join(.1)


import spirit.spiritlib as spiritlib
import ctypes

### Load Library
_spirit = spiritlib.LoadSpiritLibrary()

### Imports
from spirit.scalar import scalar
from spirit import system

import numpy as np

### ---------------------------------- Set ----------------------------------

### Set the type of Bravais lattice. Can be e.g. "sc" or "bcc"
_Set_Bravais_Lattice          = _spirit.Geometry_Set_Bravais_Lattice
_Set_Bravais_Lattice.argtypes = [ctypes.c_void_p, ctypes.c_char_p]
_Set_Bravais_Lattice.restype  = None
def setBravaisLattice(p_state, lattice, idx_image=-1, idx_chain=-1):
    _Set_Bravais_Lattice(ctypes.c_void_p(p_state), ctypes.c_char_p(lattice.encode('utf-8')))

### Set number of cells in bravais lattice directions a, b, c
_Set_N_Cells          = _spirit.Geometry_Set_N_Cells
_Set_N_Cells.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_int)]
_Set_N_Cells.restype  = None
def setNCells(p_state, n_cells=[1, 1, 1], idx_image=-1, idx_chain=-1):
    vec3 = ctypes.c_int * 3
    _Set_N_Cells(ctypes.c_void_p(p_state), vec3(*n_cells))

### Set the types of the atoms in a basis cell
_Set_Cell_Atom_Types          = _spirit.Geometry_Set_Cell_Atom_Types
_Set_Cell_Atom_Types.argtypes = [ctypes.c_void_p, ctypes.c_int, ctypes.POINTER(ctypes.c_float)]
_Set_Cell_Atom_Types.restype  = None
def setCellAtomTypes(p_state, atom_types, idx_image=-1, idx_chain=-1):
    n = len(atom_types)
    vec = ctypes.c_int * n
    _Set_Cell_Atom_Types(ctypes.c_void_p(p_state), ctypes.c_int(n), vec(*atom_types))

### Set the bravais vectors
_Set_Bravais_Vectors             = _spirit.Geometry_Set_Bravais_Vectors
_Set_Bravais_Vectors.argtypes    = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_float),
                                    ctypes.POINTER(ctypes.c_float), ctypes.POINTER(ctypes.c_float)]
_Set_Bravais_Vectors.restype     = None
def setBravaisVectors(p_state, ta=[1.0, 0.0, 0.0], tb=[0.0, 1.0, 0.0], tc=[0.0, 0.0, 1.0], idx_image=-1, idx_chain=-1):
    vec3 = ctypes.c_float * 3
    _Set_Bravais_Vectors(ctypes.c_void_p(p_state), vec3(ta), vec3(tb), vec3(tc))

### Set the overall lattice constant
_Set_Lattice_Constant             = _spirit.Geometry_Set_Lattice_Constant
_Set_Lattice_Constant.argtypes    = [ctypes.c_void_p, ctypes.c_float]
_Set_Lattice_Constant.restype     = None
def setLatticeConstant(p_state, lattice_constant, idx_image=-1, idx_chain=-1):
    _Set_Lattice_Constant(p_state, ctypes.c_float(lattice_constant))

### ---------------------------------- Get ----------------------------------

### Get Bounds
_Get_Bounds          = _spirit.Geometry_Get_Bounds
_Get_Bounds.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_float), 
                        ctypes.POINTER(ctypes.c_float), ctypes.c_int, ctypes.c_int]
_Get_Bounds.restype  = None
def Get_Bounds(p_state, idx_image=-1, idx_chain=-1):
    _min = (3*ctypes.c_float)()
    _max = (3*ctypes.c_float)()
    _Get_Bounds(ctypes.c_void_p(p_state), _min, _max, 
                ctypes.c_int(idx_image), ctypes.c_int(idx_chain))
    return [_min[i] for i in range(3)], [_max[i] for i in range(3)] 

### Get Center
_Get_Center          = _spirit.Geometry_Get_Center
_Get_Center.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_float), ctypes.c_int, ctypes.c_int]
_Get_Center.restype  = None
def Get_Center(p_state, idx_image=-1, idx_chain=-1):
    _center = (3*ctypes.c_float)()
    _Get_Center(ctypes.c_void_p(p_state), _center, ctypes.c_int(idx_image), ctypes.c_int(idx_chain))
    return [_center[i] for i in range(3)]

### Get Bravais lattice type
_Get_Bravais_Type          = _spirit.Geometry_Get_Bravais_Type
_Get_Bravais_Type.argtypes = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Get_Bravais_Type.restype  = ctypes.c_int
def Get_Bravais_Type(p_state, idx_image=-1, idx_chain=-1):
    return int(_Get_Bravais_Type(ctypes.c_void_p(p_state), ctypes.c_int(idx_image), 
                                 ctypes.c_int(idx_chain)))

### Get Bravais vectors
_Get_Bravais_Vectors          = _spirit.Geometry_Get_Bravais_Vectors
_Get_Bravais_Vectors.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_float), 
                               ctypes.POINTER(ctypes.c_float), ctypes.POINTER(ctypes.c_float), 
                               ctypes.c_int, ctypes.c_int]
_Get_Bravais_Vectors.restype  = None
def Get_Bravais_Vectors(p_state, idx_image=-1, idx_chain=-1):
    _a = (3*ctypes.c_float)()
    _b = (3*ctypes.c_float)()
    _c = (3*ctypes.c_float)()
    _Get_Bravais_Vectors(ctypes.c_void_p(p_state), _a, _b, _c, 
                       ctypes.c_int(idx_image), ctypes.c_int(idx_chain))
    return [_a[i] for i in range(3)], [_b[i] for i in range(3)], [_c[i] for i in range(3)]
    
### Get N Cells
_Get_N_Cells          = _spirit.Geometry_Get_N_Cells
_Get_N_Cells.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_int), ctypes.c_int, ctypes.c_int]
_Get_N_Cells.restype  = None
def Get_N_Cells(p_state, idx_image=-1, idx_chain=-1):
    n_cells = (3*ctypes.c_int)()
    _Get_N_Cells(ctypes.c_void_p(p_state), n_cells, ctypes.c_int(idx_image), ctypes.c_int(idx_chain))
    return [n_cells[i] for i in range(3)]

### Get Translation Vectors
_Get_Translation_Vectors          = _spirit.Geometry_Get_Translation_Vectors
_Get_Translation_Vectors.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_float), 
                                     ctypes.POINTER(ctypes.c_float), ctypes.POINTER(ctypes.c_float), 
                                     ctypes.c_int, ctypes.c_int]
_Get_Translation_Vectors.restype  = None
def Get_Translation_Vectors(p_state, idx_image=-1, idx_chain=-1):
    ta = (3*ctypes.c_float)()
    tb = (3*ctypes.c_float)()
    tc = (3*ctypes.c_float)()
    _Get_Translation_Vectors(ctypes.c_void_p(p_state), ta, tb, tc, 
                             ctypes.c_int(idx_image), ctypes.c_int(idx_chain))
    return ta, tb, tc

### Get Translation Vectors
_Get_Dimensionality          = _spirit.Geometry_Get_Dimensionality
_Get_Dimensionality.argtypes = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Get_Dimensionality.restype  = ctypes.c_int
def Get_Dimensionality(p_state, idx_image=-1, idx_chain=-1):
    return int(_Get_Dimensionality(ctypes.c_void_p(p_state), ctypes.c_int(idx_image), 
                                   ctypes.c_int(idx_chain)))

### Get Pointer to Spin Positions
# NOTE: Changing the values of the array_view one can alter the value of the data of the state
_Get_Positions            = _spirit.Geometry_Get_Positions
_Get_Positions.argtypes   = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Get_Positions.restype    = ctypes.POINTER(scalar)
def Get_Positions(p_state, idx_image=-1, idx_chain=-1):
    nos = system.Get_NOS(p_state, idx_image, idx_chain)
    ArrayType = scalar*3*nos
    Data = _Get_Positions(ctypes.c_void_p(p_state), 
                               ctypes.c_int(idx_image), ctypes.c_int(idx_chain))
    array_pointer = ctypes.cast(Data, ctypes.POINTER(ArrayType))
    array = np.frombuffer(array_pointer.contents, dtype=scalar)
    array_view = array.view()
    array_view.shape = (nos, 3)
    return array_view

### Get Pointer to atom types
# NOTE: Changing the values of the array_view one can alter the value of the data of the state
_Get_Atom_Types            = _spirit.Geometry_Get_Atom_Types
_Get_Atom_Types.argtypes   = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Get_Atom_Types.restype    = ctypes.POINTER(ctypes.c_int)
def Get_Atom_Types(p_state, idx_image=-1, idx_chain=-1):
    nos = system.Get_NOS(p_state, idx_image, idx_chain)
    ArrayType = ctypes.c_int*nos
    Data = _Get_Atom_Types(ctypes.c_void_p(p_state), 
                           ctypes.c_int(idx_image), ctypes.c_int(idx_chain))
    array_pointer = ctypes.cast(Data, ctypes.POINTER(ArrayType))
    array = np.frombuffer(array_pointer.contents, dtype=ctypes.c_int)
    array_view = array.view()
    return array_view


import spirit.spiritlib as spiritlib
import ctypes

### Load Library
_spirit = spiritlib.LoadSpiritLibrary()

### ---------------------------------- Set ----------------------------------

### Set the output file tag, which is placed in front of all output files of LLG simulations
_Set_LLG_Output_Tag          = _spirit.Parameters_Set_LLG_Output_Tag
_Set_LLG_Output_Tag.argtypes = [ctypes.c_void_p, ctypes.c_char_p,
                                ctypes.c_int, ctypes.c_int]
_Set_LLG_Output_Tag.restype  = None
def setOutputTag(p_state, tag, idx_image=-1, idx_chain=-1):
    _Set_LLG_Output_Tag(ctypes.c_void_p(p_state), ctypes.c_char_p(tag.encode('utf-8')),
                        ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set the output folder for LLG simulations
_Set_LLG_Output_Folder          = _spirit.Parameters_Set_LLG_Output_Folder
_Set_LLG_Output_Folder.argtypes = [ctypes.c_void_p, ctypes.c_char_p,
                                    ctypes.c_int, ctypes.c_int]
_Set_LLG_Output_Folder.restype  = None
def setOutputFolder(p_state, folder, idx_image=-1, idx_chain=-1):
    _Set_LLG_Output_Folder(ctypes.c_void_p(p_state), ctypes.c_char_p(folder.encode('utf-8')),
                        ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set the output configuration
_Set_LLG_Output_General          = _spirit.Parameters_Set_LLG_Output_General
_Set_LLG_Output_General.argtypes = [ctypes.c_void_p, ctypes.c_bool, ctypes.c_bool,
                                    ctypes.c_bool, ctypes.c_int, ctypes.c_int]
_Set_LLG_Output_General.restype  = None
def setOutputGeneral(p_state, any=True, initial=False, final=False, idx_image=-1, idx_chain=-1):
    _Set_LLG_Output_General(ctypes.c_void_p(p_state), ctypes.c_bool(any), ctypes.c_bool(initial),
                        ctypes.c_bool(final), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set the output configuration for energy files
_Set_LLG_Output_Energy          = _spirit.Parameters_Set_LLG_Output_Energy
_Set_LLG_Output_Energy.argtypes = [ctypes.c_void_p, ctypes.c_bool, ctypes.c_bool,
                                    ctypes.c_bool, ctypes.c_bool,
                                    ctypes.c_bool, ctypes.c_int, ctypes.c_int]
_Set_LLG_Output_Energy.restype  = None
def setOutputEnergy(p_state, step=False, archive=True, spin_resolved=False, divide_by_nos=True, add_readability_lines=True, idx_image=-1, idx_chain=-1):
    _Set_LLG_Output_Energy(ctypes.c_void_p(p_state), ctypes.c_bool(step), ctypes.c_bool(archive),
                        ctypes.c_bool(spin_resolved), ctypes.c_bool(divide_by_nos), ctypes.c_bool(add_readability_lines),
                        ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set the output configuration for energy files
_Set_LLG_Output_Configuration          = _spirit.Parameters_Set_LLG_Output_Configuration
_Set_LLG_Output_Configuration.argtypes = [ctypes.c_void_p, ctypes.c_bool, ctypes.c_bool,
                                            ctypes.c_int, ctypes.c_int, ctypes.c_int]
_Set_LLG_Output_Configuration.restype  = None
def setOutputConfiguration(p_state, step=False, archive=True, filetype=6, idx_image=-1, idx_chain=-1):
    _Set_LLG_Output_Configuration(ctypes.c_void_p(p_state), ctypes.c_bool(step), ctypes.c_bool(archive),
                        ctypes.c_int(filetype), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set LLG N Iterations
_Set_LLG_N_Iterations             = _spirit.Parameters_Set_LLG_N_Iterations
_Set_LLG_N_Iterations.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int,
                                     ctypes.c_int, ctypes.c_int]
_Set_LLG_N_Iterations.restype     = None
def setIterations(p_state, n_iterations, n_iterations_log, idx_image=-1, idx_chain=-1):
    _Set_LLG_N_Iterations(ctypes.c_void_p(p_state), ctypes.c_int(n_iterations),
                          ctypes.c_int(n_iterations_log), ctypes.c_int(idx_image), 
                          ctypes.c_int(idx_chain))

### Set LLG Direct Minimization
_Set_LLG_Direct_Minimization            = _spirit.Parameters_Set_LLG_Direct_Minimization
_Set_LLG_Direct_Minimization.argtypes   = [ctypes.c_void_p, ctypes.c_bool,
                                           ctypes.c_int, ctypes.c_int ]
_Set_LLG_Direct_Minimization.restype    = None
def setDirectMinimization(p_state, use_minimization, idx_image=-1, idx_chain=-1):
    _Set_LLG_Direct_Minimization(ctypes.c_void_p(p_state), ctypes.c_bool(use_minimization), 
                                 ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

_Set_LLG_Convergence            = _spirit.Parameters_Set_LLG_Convergence
_Set_LLG_Convergence.argtypes   = [ctypes.c_void_p, ctypes.c_float, ctypes.c_int, ctypes.c_int ]
_Set_LLG_Convergence.restype    = None
def setConvergence(p_state, convergence, idx_image=-1, idx_chain=-1):
    _Set_LLG_Convergence(ctypes.c_void_p(p_state), ctypes.c_float(convergence),
                         ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set LLG Time Step
_Set_LLG_Time_Step             = _spirit.Parameters_Set_LLG_Time_Step
_Set_LLG_Time_Step.argtypes    = [ctypes.c_void_p, ctypes.c_float, ctypes.c_int, ctypes.c_int]
_Set_LLG_Time_Step.restype     = None
def setTimeStep(p_state, dt, idx_image=-1, idx_chain=-1):
    _Set_LLG_Time_Step(p_state, ctypes.c_float(dt), idx_image, idx_chain)

### Set LLG Damping
_Set_LLG_Damping             = _spirit.Parameters_Set_LLG_Damping
_Set_LLG_Damping.argtypes    = [ctypes.c_void_p, ctypes.c_float, ctypes.c_int, ctypes.c_int]
_Set_LLG_Damping.restype     = None
def setDamping(p_state, damping, idx_image=-1, idx_chain=-1):
    _Set_LLG_Damping(ctypes.c_void_p(p_state), ctypes.c_float(damping),
                     ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set spin transfer torque
# Note: direction must be  list
_Set_LLG_STT             = _spirit.Parameters_Set_LLG_STT
_Set_LLG_STT.argtypes    = [ctypes.c_void_p, ctypes.c_bool, ctypes.c_float,
                            ctypes.POINTER(3*ctypes.c_float), ctypes.c_int, ctypes.c_int]
_Set_LLG_STT.restype     = None
def setSTT(p_state, use_gradient, magnitude, direction, idx_image=-1, idx_chain=-1):
    vec3 = ctypes.c_float * 3
    direction = vec3(*direction)
    ctypes.cast( direction, ctypes.POINTER(vec3))
    _Set_LLG_STT(ctypes.c_void_p(p_state), ctypes.c_bool(use_gradient), 
                 ctypes.c_float(magnitude), direction, ctypes.c_int(idx_image), 
                 ctypes.c_int(idx_chain))

### Set Temperature
_Set_LLG_Temperature             = _spirit.Parameters_Set_LLG_Temperature
_Set_LLG_Temperature.argtypes    = [ctypes.c_void_p, ctypes.c_float, ctypes.c_int, ctypes.c_int]
_Set_LLG_Temperature.restype     = None
_Set_LLG_Temperature_Gradient             = _spirit.Parameters_Set_LLG_Temperature_Gradient
_Set_LLG_Temperature_Gradient.argtypes    = [ctypes.c_void_p, ctypes.c_float, ctypes.POINTER(3*ctypes.c_float),
                                             ctypes.c_int, ctypes.c_int]
_Set_LLG_Temperature_Gradient.restype     = None
def setTemperature(p_state, temperature, gradient_inclination=0, gradient_direction=[1.0,0.0,0.0], idx_image=-1, idx_chain=-1):
    _Set_LLG_Temperature(ctypes.c_void_p(p_state), ctypes.c_float(temperature), 
                         ctypes.c_int(idx_image), ctypes.c_int(idx_chain))
    vec3 = ctypes.c_float * 3
    gradient_direction = vec3(*gradient_direction)
    ctypes.cast( gradient_direction, ctypes.POINTER(vec3))
    _Set_LLG_Temperature_Gradient(ctypes.c_void_p(p_state), ctypes.c_float(gradient_inclination), gradient_direction,
                                  ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### ---------------------------------- Get ----------------------------------

### Get LLG N Iterations
_Get_LLG_N_Iterations             = _spirit.Parameters_Get_LLG_N_Iterations
_Get_LLG_N_Iterations.argtypes    = [ctypes.c_void_p, ctypes.POINTER( ctypes.c_int ),
                                     ctypes.POINTER( ctypes.c_int ), ctypes.c_int, ctypes.c_int]
_Get_LLG_N_Iterations.restype     = None
def getIterations(p_state, idx_image=-1, idx_chain=-1):
    n_iterations = ctypes.c_int()
    n_iterations_log = ctypes.c_int()
    _Get_LLG_N_Iterations(p_state, ctypes.pointer(n_iterations), ctypes.pointer(n_iterations_log),
                          ctypes.c_int(idx_image), ctypes.c_int(idx_chain))
    return int(n_iterations.value), int(n_iterations_log.value)

### Get LLG Direct Minimization
_Get_LLG_Direct_Minimization            = _spirit.Parameters_Get_LLG_Direct_Minimization
_Get_LLG_Direct_Minimization.argtypes   = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int ]
_Get_LLG_Direct_Minimization.restype    = ctypes.c_bool
def getDirectMinimization(p_state, idx_image=-1, idx_chain=-1):
    return float(_Get_LLG_Direct_Minimization(ctypes.c_void_p(p_state),
                                              ctypes.c_int(idx_image), ctypes.c_int(idx_chain)))

### Get LLG Convergence
_Get_LLG_Convergence            = _spirit.Parameters_Get_LLG_Convergence
_Get_LLG_Convergence.argtypes   = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int ]
_Get_LLG_Convergence.restype    = ctypes.c_float
def getConvergence(p_state, idx_image=-1, idx_chain=-1):
    return float(_Get_LLG_Convergence(ctypes.c_void_p(p_state), ctypes.c_int(idx_image),
                                      ctypes.c_int(idx_chain)))

### Get LLG Time Step in [psec]
_Get_LLG_Time_Step             = _spirit.Parameters_Get_LLG_Time_Step
_Get_LLG_Time_Step.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Get_LLG_Time_Step.restype     = ctypes.c_float
def getTimeStep(p_state, idx_image=-1, idx_chain=-1):
    return float(_Get_LLG_Time_Step(ctypes.c_void_p(p_state), ctypes.c_int(idx_image), 
                                    ctypes.c_int(idx_chain)))

### Get LLG Damping
_Get_LLG_Damping             = _spirit.Parameters_Get_LLG_Damping
_Get_LLG_Damping.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Get_LLG_Damping.restype     = ctypes.c_float
def getDamping(p_state, idx_image=-1, idx_chain=-1):
    return float(_Get_LLG_Damping(ctypes.c_void_p(p_state), ctypes.c_int(idx_image),
                            ctypes.c_int(idx_chain)))

### Get LLG STT
_Get_LLG_STT             = _spirit.Parameters_Get_LLG_STT
_Get_LLG_STT.argtypes    = [ctypes.c_void_p, ctypes.POINTER( ctypes.c_bool ), 
                            ctypes.POINTER( ctypes.c_float ), ctypes.POINTER(ctypes.c_float), 
                            ctypes.c_int, ctypes.c_int]
_Get_LLG_STT.restype     = None
def getSTT(p_state, idx_image=-1, idx_chain=-1):
    direction = (3*ctypes.c_float)()
    use_gradient = ctypes.c_bool()
    magnitude = ctypes.c_float()
    _Get_LLG_STT(ctypes.c_void_p(p_state), ctypes.pointer(use_gradient), 
                 ctypes.pointer(magnitude), direction, ctypes.c_int(idx_image), 
                 ctypes.c_int(idx_chain))
    return float(magnitude.value), [float(i) for i in direction], bool(use_gradient)
    
### Get Temperature
_Get_LLG_Temperature             = _spirit.Parameters_Get_LLG_Temperature
_Get_LLG_Temperature.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Get_LLG_Temperature.restype     = ctypes.c_float
_Get_LLG_Temperature_Gradient             = _spirit.Parameters_Get_LLG_Temperature_Gradient
_Get_LLG_Temperature_Gradient.argtypes    = [ctypes.c_void_p, ctypes.POINTER( ctypes.c_float ),
                                             ctypes.POINTER(ctypes.c_float), ctypes.c_int, ctypes.c_int]
_Get_LLG_Temperature_Gradient.restype     = None
def getTemperature(p_state, idx_image=-1, idx_chain=-1):
    temperature = float(_Get_LLG_Temperature(ctypes.c_void_p(p_state), ctypes.c_int(idx_image),
                                             ctypes.c_int(idx_chain)))
    gradient_inclination = ctypes.c_float()
    gradient_direction = (3*ctypes.c_float)()
    _Get_LLG_Temperature_Gradient(ctypes.c_void_p(p_state), ctypes.pointer(gradient_inclination),
                                  gradient_direction, ctypes.c_int(idx_image), ctypes.c_int(idx_chain))
    return temperature, gradient_inclination, gradient_direction



import spirit.spiritlib as spiritlib
import ctypes

### Load Library
_spirit = spiritlib.LoadSpiritLibrary()


### Domain (homogeneous) configuration
_Domain             = _spirit.Configuration_Domain
_Domain.argtypes    = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_float), 
                       ctypes.POINTER(ctypes.c_float), ctypes.POINTER(ctypes.c_float), 
                       ctypes.c_float, ctypes.c_float, ctypes.c_bool, ctypes.c_int, ctypes.c_int]
_Domain.restype     = None
def Domain(p_state, dir, pos=[0,0,0], border_rectangular=[-1,-1,-1], border_cylindrical=-1, 
           border_spherical=-1, inverted=False, idx_image=-1, idx_chain=-1):
    vec3 = ctypes.c_float * 3
    _Domain(ctypes.c_void_p(p_state) , vec3(*dir), vec3(*pos), vec3(*border_rectangular), 
            ctypes.c_float(border_cylindrical), ctypes.c_float(border_spherical), 
            ctypes.c_bool(inverted), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))


### All spins in +z direction
_PlusZ             = _spirit.Configuration_PlusZ
_PlusZ.argtypes    = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_float), 
                      ctypes.POINTER(ctypes.c_float), ctypes.c_float, ctypes.c_float, ctypes.c_bool,
                      ctypes.c_int, ctypes.c_int]
_PlusZ.restype     = None
def PlusZ(p_state, pos=[0.0,0.0,0.0], border_rectangular=[-1.0,-1.0,-1.0], border_cylindrical=-1.0, 
          border_spherical=-1.0, inverted=False, idx_image=-1, idx_chain=-1):
    vec3 = ctypes.c_float * 3
    _PlusZ(ctypes.c_void_p(p_state) , vec3(*pos), vec3(*border_rectangular), 
           ctypes.c_float(border_cylindrical), ctypes.c_float(border_spherical), 
           ctypes.c_bool(inverted), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))


### All spins in -z direction
_MinusZ             = _spirit.Configuration_MinusZ
_MinusZ.argtypes    = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_float), 
                       ctypes.POINTER(ctypes.c_float), ctypes.c_float, ctypes.c_float, 
                       ctypes.c_bool, ctypes.c_int, ctypes.c_int]
_MinusZ.restype     = None
def MinusZ(p_state, pos=[0,0,0], border_rectangular=[-1,-1,-1], border_cylindrical=-1, 
          border_spherical=-1, inverted=False, idx_image=-1, idx_chain=-1):
    vec3 = ctypes.c_float * 3
    _MinusZ(ctypes.c_void_p(p_state) , vec3(*pos), vec3(*border_rectangular), 
            ctypes.c_float(border_cylindrical), ctypes.c_float(border_spherical), 
            ctypes.c_bool(inverted), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Random configuration
_Random             = _spirit.Configuration_Random
_Random.argtypes    = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_float), 
                       ctypes.POINTER(ctypes.c_float), ctypes.c_float, ctypes.c_float, 
                       ctypes.c_bool, ctypes.c_bool, ctypes.c_int, ctypes.c_int]
_Random.restype     = None
def Random(p_state, pos=[0,0,0], border_rectangular=[-1,-1,-1], border_cylindrical=-1, 
           border_spherical=-1, inverted=False, idx_image=-1, idx_chain=-1):
    vec3 = ctypes.c_float * 3
    _Random(ctypes.c_void_p(p_state) , vec3(*pos), vec3(*border_rectangular), 
            ctypes.c_float(border_cylindrical), ctypes.c_float(border_spherical), 
            ctypes.c_bool(inverted), False, ctypes.c_int(idx_image), ctypes.c_int(idx_chain))


### Add temperature-scaled random noise to configuration
_Add_Noise_Temperature             = _spirit.Configuration_Add_Noise_Temperature
_Add_Noise_Temperature.argtypes    = [ctypes.c_void_p, ctypes.c_float, 
                                      ctypes.POINTER(ctypes.c_float), ctypes.POINTER(ctypes.c_float), 
                                      ctypes.c_float, ctypes.c_float, ctypes.c_bool, ctypes.c_bool, 
                                      ctypes.c_int, ctypes.c_int]
_Add_Noise_Temperature.restype     = None
def Add_Noise_Temperature(p_state, temperature, pos=[0,0,0], border_rectangular=[-1,-1,-1], 
                          border_cylindrical=-1, border_spherical=-1, inverted=False, 
                          idx_image=-1, idx_chain=-1):
    vec3 = ctypes.c_float * 3
    _Add_Noise_Temperature(ctypes.c_void_p(p_state) , ctypes.c_float(temperature), vec3(*pos), 
                           vec3(*border_rectangular), ctypes.c_float(border_cylindrical), 
                           ctypes.c_float(border_spherical), ctypes.c_bool(inverted), False, 
                           ctypes.c_int(idx_image), ctypes.c_int(idx_chain))


### Skyrmion configuration
_Skyrmion             = _spirit.Configuration_Skyrmion
_Skyrmion.argtypes    = [ctypes.c_void_p, ctypes.c_float, ctypes.c_float, ctypes.c_float, 
                         ctypes.c_bool, ctypes.c_bool, ctypes.c_bool, ctypes.POINTER(ctypes.c_float), 
                         ctypes.POINTER(ctypes.c_float), ctypes.c_float, ctypes.c_float, ctypes.c_bool, 
                         ctypes.c_int, ctypes.c_int]
_Skyrmion.restype     = None
def Skyrmion(p_state, radius, order=1, phase=1, upDown=False, achiral=False, rightleft=False, 
             pos=[0,0,0], border_rectangular=[-1,-1,-1], border_cylindrical=-1, border_spherical=-1, 
             inverted=False, idx_image=-1, idx_chain=-1):
    vec3 = ctypes.c_float * 3
    _Skyrmion(ctypes.c_void_p(p_state) , ctypes.c_float(radius), ctypes.c_float(order), 
              ctypes.c_float(phase), ctypes.c_bool(upDown), ctypes.c_bool(achiral), 
              ctypes.c_bool(rightleft), vec3(*pos), vec3(*border_rectangular), 
              ctypes.c_float(border_cylindrical), ctypes.c_float(border_spherical), 
              ctypes.c_bool(inverted), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Hopfion configuration
_Hopfion             = _spirit.Configuration_Hopfion
_Hopfion.argtypes    = [ctypes.c_void_p, ctypes.c_float, ctypes.c_int, ctypes.POINTER(ctypes.c_float), 
                        ctypes.POINTER(ctypes.c_float), ctypes.c_float, ctypes.c_float, ctypes.c_bool, 
                        ctypes.c_int, ctypes.c_int]
_Hopfion.restype     = None
def Hopfion(p_state, radius, order=1, pos=[0,0,0], border_rectangular=[-1,-1,-1], 
            border_cylindrical=-1, border_spherical=-1, inverted=False, idx_image=-1, idx_chain=-1):
    vec3 = ctypes.c_float * 3
    _Hopfion(ctypes.c_void_p(p_state) , ctypes.c_float(radius), ctypes.c_int(order), 
             vec3(*pos), vec3(*border_rectangular), ctypes.c_float(border_cylindrical), 
             ctypes.c_float(border_spherical), ctypes.c_bool(inverted), 
             ctypes.c_int(idx_image), ctypes.c_int(idx_chain))


### Spin Spiral configuration
_SpinSpiral             = _spirit.Configuration_SpinSpiral
_SpinSpiral.argtypes    = [ctypes.c_void_p, ctypes.c_char_p, ctypes.POINTER(ctypes.c_float), 
                           ctypes.POINTER(ctypes.c_float), ctypes.c_float, 
                           ctypes.POINTER(ctypes.c_float), ctypes.POINTER(ctypes.c_float), 
                           ctypes.c_float, ctypes.c_float, ctypes.c_bool, 
                           ctypes.c_int, ctypes.c_int]
_SpinSpiral.restype     = None
def SpinSpiral(p_state, direction_type, q_vector, axis, theta, pos=[0,0,0], 
               border_rectangular=[-1,-1,-1], border_cylindrical=-1, border_spherical=-1, 
               inverted=False, idx_image=-1, idx_chain=-1):
    vec3 = ctypes.c_float * 3
    _SpinSpiral(ctypes.c_void_p(p_state) , ctypes.c_char_p(direction_type.encode('utf-8')), 
                vec3(*q_vector), vec3(*axis), ctypes.c_float(theta), vec3(*pos), 
                vec3(*border_rectangular), ctypes.c_float(border_cylindrical), 
                ctypes.c_float(border_spherical), ctypes.c_bool(inverted), 
                ctypes.c_int(idx_image), ctypes.c_int(idx_chain))



import spirit.spiritlib as spiritlib
import ctypes

### Load Library
_spirit = spiritlib.LoadSpiritLibrary()

### TODO:
#Hamiltonian_Set_mu_s
#Hamiltonian_Set_DMI
#Hamiltonian_Set_BQE
#Hamiltonian_Set_FSC
#all getters (including name!)

### Set external magnetic field
_Set_Field             = _spirit.Hamiltonian_Set_Field
_Set_Field.argtypes    = [ctypes.c_void_p, ctypes.c_float, ctypes.POINTER(ctypes.c_float), 
                          ctypes.c_int, ctypes.c_int]
_Set_Field.restype     = None
def Set_Field(p_state, magnitude, direction, idx_image=-1, idx_chain=-1):
    vec3 = ctypes.c_float * 3
    _Set_Field(ctypes.c_void_p(p_state), ctypes.c_float(magnitude), vec3(*direction), 
               ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set anisotropy
_Set_Anisotropy             = _spirit.Hamiltonian_Set_Anisotropy
_Set_Anisotropy.argtypes    = [ctypes.c_void_p, ctypes.c_float, ctypes.POINTER(ctypes.c_float), 
                               ctypes.c_int, ctypes.c_int]
_Set_Anisotropy.restype     = None
def Set_Anisotropy(p_state, magnitude, direction, idx_image=-1, idx_chain=-1):
    vec3 = ctypes.c_float * 3
    _Set_Anisotropy(ctypes.c_void_p(p_state), ctypes.c_float(magnitude), vec3(*direction), 
                    ctypes.c_int(idx_image), ctypes.c_int(idx_chain))



import spirit.spiritlib as spiritlib
import ctypes

### Load Library
_spirit = spiritlib.LoadSpiritLibrary()

### ---------------------------------- Set ----------------------------------

### Set the output file tag, which is placed in front of all output files of GNEB simulations
_Set_GNEB_Output_Tag          = _spirit.Parameters_Set_GNEB_Output_Tag
_Set_GNEB_Output_Tag.argtypes = [ctypes.c_void_p, ctypes.c_char_p,
                                ctypes.c_int, ctypes.c_int]
_Set_GNEB_Output_Tag.restype  = None
def setOutputTag(p_state, tag, idx_image=-1, idx_chain=-1):
    _Set_GNEB_Output_Tag(ctypes.c_void_p(p_state), ctypes.c_char_p(tag.encode('utf-8')),
                        ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set the output folder for GNEB simulations
_Set_GNEB_Output_Folder          = _spirit.Parameters_Set_GNEB_Output_Folder
_Set_GNEB_Output_Folder.argtypes = [ctypes.c_void_p, ctypes.c_char_p,
                                    ctypes.c_int, ctypes.c_int]
_Set_GNEB_Output_Folder.restype  = None
def setOutputFolder(p_state, folder, idx_image=-1, idx_chain=-1):
    _Set_GNEB_Output_Folder(ctypes.c_void_p(p_state), ctypes.c_char_p(folder.encode('utf-8')),
                        ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set the output configuration
_Set_GNEB_Output_General          = _spirit.Parameters_Set_GNEB_Output_General
_Set_GNEB_Output_General.argtypes = [ctypes.c_void_p, ctypes.c_bool, ctypes.c_bool,
                                    ctypes.c_bool, ctypes.c_int, ctypes.c_int]
_Set_GNEB_Output_General.restype  = None
def setOutputGeneral(p_state, any=True, initial=False, final=False, idx_image=-1, idx_chain=-1):
    _Set_GNEB_Output_General(ctypes.c_void_p(p_state), ctypes.c_bool(any), ctypes.c_bool(initial),
                        ctypes.c_bool(final), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set the output configuration for energy files
_Set_GNEB_Output_Energies          = _spirit.Parameters_Set_GNEB_Output_Energies
_Set_GNEB_Output_Energies.argtypes = [ctypes.c_void_p, ctypes.c_bool, ctypes.c_bool,
                                    ctypes.c_bool, ctypes.c_bool, ctypes.c_int, ctypes.c_int]
_Set_GNEB_Output_Energies.restype  = None
def setOutputEnergy(p_state, step=True, interpolated=True, divide_by_nos=True, add_readability_lines=True, idx_image=-1, idx_chain=-1):
    _Set_GNEB_Output_Energies(ctypes.c_void_p(p_state), ctypes.c_bool(step), ctypes.c_bool(interpolated),
                        ctypes.c_bool(divide_by_nos), ctypes.c_bool(add_readability_lines),
                        ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set the output configuration for energy files
_Set_GNEB_Output_Chain          = _spirit.Parameters_Set_GNEB_Output_Chain
_Set_GNEB_Output_Chain.argtypes = [ctypes.c_void_p, ctypes.c_bool, ctypes.c_int, ctypes.c_int, ctypes.c_int]
_Set_GNEB_Output_Chain.restype  = None
def setOutputConfiguration(p_state, step=False, filetype=6, idx_image=-1, idx_chain=-1):
    _Set_GNEB_Output_Chain(ctypes.c_void_p(p_state), ctypes.c_bool(step), ctypes.c_int(filetype),
                        ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set GNEB N Iterations
_Set_GNEB_N_Iterations             = _spirit.Parameters_Set_GNEB_N_Iterations
_Set_GNEB_N_Iterations.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int,
                                      ctypes.c_int, ctypes.c_int]
_Set_GNEB_N_Iterations.restype     = None
def setIterations(p_state, n_iterations, n_iterations_log, idx_image=-1, idx_chain=-1):
    _Set_GNEB_N_Iterations(ctypes.c_void_p(p_state), ctypes.c_int(n_iterations),
                           ctypes.c_int(n_iterations_log), ctypes.c_int(idx_image),
                           ctypes.c_int(idx_chain))

### Set GNEB convergence
_Set_GNEB_Convergence           = _spirit.Parameters_Set_GNEB_Convergence
_Set_GNEB_Convergence.argtypes  = [ctypes.c_void_p, ctypes.c_float, ctypes.c_int, ctypes.c_int]
_Set_GNEB_Convergence.restype   = None
def setConvergence(p_state, convergence, idx_image=-1, idx_chain=-1):
    _Set_GNEB_Convergence(ctypes.c_void_p(p_state), ctypes.c_float(convergence),
                          ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set GNEB Spring Constant
_Set_GNEB_Spring_Constant           = _spirit.Parameters_Set_GNEB_Spring_Constant
_Set_GNEB_Spring_Constant.argtypes  = [ctypes.c_void_p, ctypes.c_float, ctypes.c_int, ctypes.c_int]
_Set_GNEB_Spring_Constant.restype   = None
def setSpringConstant(p_state, c_spring, idx_image=-1, idx_chain=-1):
    _Set_GNEB_Spring_Constant(ctypes.c_void_p(p_state), ctypes.c_float(c_spring),
                              ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set GNEB climbing and falling images
_Set_GNEB_Climbing_Falling             = _spirit.Parameters_Set_GNEB_Climbing_Falling
_Set_GNEB_Climbing_Falling.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int, ctypes.c_int]
_Set_GNEB_Climbing_Falling.restype     = None
def setClimbingFalling(p_state, image_type, idx_image=-1, idx_chain=-1):
    _Set_GNEB_Climbing_Falling(ctypes.c_void_p(p_state), ctypes.c_int(image_type),
                               ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Set GNEB climbing and falling images
_Set_GNEB_Image_Type_Automatically             = _spirit.Parameters_Set_GNEB_Image_Type_Automatically
_Set_GNEB_Image_Type_Automatically.argtypes    = [ctypes.c_void_p, ctypes.c_int]
_Set_GNEB_Image_Type_Automatically.restype     = None
def setImageTypeAutomatically(p_state, idx_chain=-1):
    _Set_GNEB_Image_Type_Automatically(ctypes.c_void_p(p_state), ctypes.c_int(idx_chain))

### ---------------------------------- Get ----------------------------------

### Get GNEB N Iterations
_Get_GNEB_N_Iterations             = _spirit.Parameters_Get_GNEB_N_Iterations
_Get_GNEB_N_Iterations.argtypes    = [ctypes.c_void_p, ctypes.POINTER( ctypes.c_int ),
                                      ctypes.POINTER( ctypes.c_int ), ctypes.c_int]
_Get_GNEB_N_Iterations.restype     = None
def getIterations(p_state, idx_image=-1, idx_chain=-1):
    n_iterations = ctypes.c_int()
    n_iterations_log = ctypes.c_int()
    _Get_GNEB_N_Iterations(ctypes.c_void_p(p_state), ctypes.pointer(n_iterations),
                           ctypes.pointer(n_iterations_log), ctypes.c_int(idx_image),
                           ctypes.c_int(idx_chain) )
    return int(n_iterations.value), int(n_iterations_log.value)

### Get GNEB convergence
_Get_GNEB_Convergence           = _spirit.Parameters_Get_GNEB_Convergence
_Get_GNEB_Convergence.argtypes  = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Get_GNEB_Convergence.restype   = ctypes.c_float
def getConvergence(p_state, idx_image=-1, idx_chain=-1):
    return float( _Get_GNEB_Convergence(ctypes.c_void_p(p_state), 
                                        ctypes.c_int(idx_image), ctypes.c_int(idx_chain)))

### Get GNEB Spring Constant
_Get_GNEB_Spring_Constant             = _spirit.Parameters_Get_GNEB_Spring_Constant
_Get_GNEB_Spring_Constant.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Get_GNEB_Spring_Constant.restype     = ctypes.c_float
def getSpringConstant(p_state, idx_image=-1, idx_chain=-1):
    return float(_Get_GNEB_Spring_Constant(ctypes.c_void_p(p_state), 
                                           ctypes.c_int(idx_image), ctypes.c_int(idx_chain)))

### Get GNEB climbing and falling images
_Get_GNEB_Climbing_Falling             = _spirit.Parameters_Get_GNEB_Climbing_Falling
_Get_GNEB_Climbing_Falling.argtypes    = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
_Get_GNEB_Climbing_Falling.restype     = ctypes.c_int
def getClimbingFalling(p_state, idx_image=-1, idx_chain=-1):
    return int(_Get_GNEB_Climbing_Falling(ctypes.c_void_p(p_state), 
                                          ctypes.c_int(idx_image), ctypes.c_int(idx_chain)))

### Get GNEB number of energy interpolations
_Get_GNEB_N_Energy_Interpolations             = _spirit.Parameters_Get_GNEB_N_Energy_Interpolations
_Get_GNEB_N_Energy_Interpolations.argtypes    = [ctypes.c_void_p, ctypes.c_int]
_Get_GNEB_N_Energy_Interpolations.restype     = ctypes.c_int
def getEnergyInterpolations(p_state, idx_chain=-1):
    return int(_Get_GNEB_N_Energy_Interpolations(ctypes.c_void_p(p_state), ctypes.c_int(idx_chain)))


import spirit.spiritlib as spiritlib
import ctypes

### Load Library
_spirit = spiritlib.LoadSpiritLibrary()


### Send a Log message
_Send          = _spirit.Log_Send
_Send.argtypes = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int, ctypes.c_char_p, 
                  ctypes.c_int, ctypes.c_int]
_Send.restype  = None
def Send(p_state, level, sender, message, idx_image=-1, idx_chain=-1):
    _Send(ctypes.c_void_p(p_state), ctypes.c_int(level), ctypes.c_int(sender), 
          ctypes.c_char_p(message.encode('utf-8')), ctypes.c_int(idx_image), ctypes.c_int(idx_chain))

### Append Log to file
_Append          = _spirit.Log_Append
_Append.argtypes = [ctypes.c_void_p]
_Append.restype  = None
def Append(p_state):
    _Append(ctypes.c_void_p(p_state))

### Get number of Log entries
_Get_N_Entries          = _spirit.Log_Get_N_Entries
_Get_N_Entries.argtypes = [ctypes.c_void_p]
_Get_N_Entries.restype  = None
def GetNEntries(p_state):
    return int(_Get_N_Entries(ctypes.c_void_p(p_state)))

### Get number of error messages
_Get_N_Errors          = _spirit.Log_Get_N_Errors
_Get_N_Errors.argtypes = [ctypes.c_void_p]
_Get_N_Errors.restype  = None
def GetNErrors(p_state):
    return int(_Get_N_Errors(ctypes.c_void_p(p_state)))

### Get number of warning messages
_Get_N_Warnings          = _spirit.Log_Get_N_Warnings
_Get_N_Warnings.argtypes = [ctypes.c_void_p]
_Get_N_Warnings.restype  = None
def GetNWarnings(p_state):
    return int(_Get_N_Warnings(ctypes.c_void_p(p_state)))

### Set the tag in front of the Log file
_Set_Output_File_Tag          = _spirit.Log_Set_Output_File_Tag
_Set_Output_File_Tag.argtypes = [ctypes.c_char_p]
_Set_Output_File_Tag.restype  = None
def SetOutputFileTag(p_state, tag):
    _Set_Output_File_Tag(ctypes.c_void_p(p_state), ctypes.c_char_p(tag.encode('utf-8')))

### Set the tag in front of the Log file
_Set_Output_Folder          = _spirit.Log_Set_Output_Folder
_Set_Output_Folder.argtypes = [ctypes.c_char_p]
_Set_Output_Folder.restype  = None
def SetOutputFolder(p_state, tag):
    _Set_Output_Folder(ctypes.c_void_p(p_state), ctypes.c_char_p(tag.encode('utf-8')))

### Set whether the Log is output to the console
_Set_Output_To_Console          = _spirit.Log_Set_Output_To_Console
_Set_Output_To_Console.argtypes = [ctypes.c_void_p, ctypes.c_bool]
_Set_Output_To_Console.restype  = None
def SetOutputToConsole(p_state, b):
    _Set_Output_To_Console(ctypes.c_void_p(p_state), ctypes.c_bool(b))

### Set the level up to which the Log is output to the console
_Set_Output_Console_Level          = _spirit.Log_Set_Output_Console_Level
_Set_Output_Console_Level.argtypes = [ctypes.c_void_p, ctypes.c_int]
_Set_Output_Console_Level.restype  = None
def SetOutputConsoleLevel(p_state, level):
    _Set_Output_Console_Level(ctypes.c_void_p(p_state), ctypes.c_int(level))

### Set whether the Log is output to a file
_Set_Output_To_File          = _spirit.Log_Set_Output_To_File
_Set_Output_To_File.argtypes = [ctypes.c_void_p, ctypes.c_bool]
_Set_Output_To_File.restype  = None
def SetOutputToFile(p_state, b):
    _Set_Output_To_File(ctypes.c_void_p(p_state), ctypes.c_bool(b))

### Set the level up to which the Log is output to a file
_Set_Output_File_Level          = _spirit.Log_Set_Output_File_Level
_Set_Output_File_Level.argtypes = [ctypes.c_void_p, ctypes.c_int]
_Set_Output_File_Level.restype  = None
def SetOutputFileLevel(p_state, level):
    _Set_Output_File_Level(ctypes.c_void_p(p_state), ctypes.c_int(level))

### Returns whether the Log is output to the console
_Get_Output_To_Console          = _spirit.Log_Get_Output_To_Console
_Get_Output_To_Console.argtypes = [ctypes.c_void_p]
_Get_Output_To_Console.restype  = ctypes.c_bool
def GetOutputToConsole(p_state):
    return bool(_Get_Output_To_Console(ctypes.c_void_p(p_state)))

### Returns the level up to which the Log is output to the console
_Get_Output_Console_Level          = _spirit.Log_Get_Output_Console_Level
_Get_Output_Console_Level.argtypes = [ctypes.c_void_p]
_Get_Output_Console_Level.restype  = ctypes.c_int
def GetOutputConsoleLevel(p_state):
    return int(_Get_Output_Console_Level(ctypes.c_void_p(p_state)))

### Returns whether the Log is output to a file
_Get_Output_To_File          = _spirit.Log_Get_Output_To_File
_Get_Output_To_File.argtypes = [ctypes.c_void_p]
_Get_Output_To_File.restype  = ctypes.c_bool
def GetOutputToFile(p_state):
    return bool(_Get_Output_To_File(ctypes.c_void_p(p_state)))

### Returns the level up to which the Log is output to a file
_Get_Output_File_Level          = _spirit.Log_Get_Output_File_Level
_Get_Output_File_Level.argtypes = [ctypes.c_void_p]
_Get_Output_File_Level.restype  = ctypes.c_int
def GetOutputFileLevel(p_state):
    return int(_Get_Output_File_Level(ctypes.c_void_p(p_state)))